/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author:Tara Madhyastha (tara@cs.uiuc.edu) 
 *
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, and by a grant
 * from the Digital Equipment Corporation External Research Program.
 *
 */

#include "EnumType.h"

/* virtual */
void EnumType::binSend(int sfd)
{
  fd = sfd;
  send(ival);
  for(int i=0; i < ival; i++) {
    send((char *)enumlist.getElement(i));
  }
}

/* virtual */
void EnumType::binReceive(int sfd)
{
  fd = sfd;
  receive(ival);
  for( int i = 0; i < ival; i++ ) {
    enumlist.addElement(recString());
  }
}

/* virtual */
char* EnumType::print()
{
  char *s = new char[256];

  int i, len=256, ssz=256;
  char n[256];
  strcpy(s, "<\n");
  for(i=0; i < ival; i++) {
    sprintf(n, "%d: %s\n", i, (char *)enumlist.getElement(i));
    len -= strlen(n);
    if (len < 0) {
      s = realloc(s, ssz+=256);
      len+=256;
    }
    strcat(s, n);
  }
  strcat(s, ">");
  return (s);
}
 

/* virtual */
void EnumType::initFromFile( char *filename )
{
  filebuf fb;
  char nl, tmpbuf[256], *foo;

  if (fb.open(filename, input) == 0 ) {
    form("Error: EnumVal::InitEnumFromFile: filename=%s, %s\n", filename,
         sys_errlist[errno]);
    return;
  }
  istream from(&fb);

  while (from.get(tmpbuf, 256, '\n') != NULL) {
    foo = new char[strlen(tmpbuf) + 1];
    strcpy(foo, tmpbuf);
    enumlist.addElement(foo);
    from.get(nl);
  }
  ival = enumlist.numberArgs();
}

void EnumType::printOn(ostream &os)
{
  os << ival;
  for(int i=0; i < ival; i++) {
    os << (char *)enumlist.getElement(i);
  }
}

