/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987, 1988, 1989, 1990, 1991, 1992
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Tara Madhyastha (tara@cs.uiuc.edu)
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, and by a grant
 * from the Digital Equipment Corporation External Research Program.
 *
 */
/*
 * Message.h: Implements sonic messages.
 *
 */

#pragma once
#ifndef Message_h
#define Message_h

#include "ValList.h"
#include "IndexList.h"
#include "DeviceSpecificDefs.h"


extern DEVICETYPE _mySoundDevice;


class Message : public Data {
/*
 * A Message is the basic unit of communication between a local process and
 * a remote sound device. It is a single command to the sound device.
 */

private:
  int MsgNumber = -1;
  char *MsgName = NULL;
  IndexList arguments;
  PMF pmf;
  ValList vl;

public:
  /* Method Message: The Message constructor. */
  Message(char *msgname, IndexList &args, PMF mf);
  Message() { } ;

  /* Method ~Message: The Message destructor. */
  ~Message();

  /* Method getIndexList: Return the IndexList describing message arguments */
  IndexList& getIndexList() { return arguments; } ;
  
  /* Method getValList: Return the ValList describing message arguments */
        ValList&  getValList() { return vl; } ;

  /* Method getName: Return the message name */ 
  char* getName() { return MsgName; } ;

  /* Method binReceive: Read a binary representation of Message from sfd */
  void binReceive(int sfd) { vl.clear(); vl.binReceiveValues(sfd); } ;

  /* Method binSend: Write a binary representation of Message to sfd */
  void binSend(int sfd) { vl.binSendValues(sfd); } ;

  /* Method binReceiveIndexList: Receive a binary representation of  * 
   *                         IndexList describing message arguments  */
        void binReceiveIndexList(int sfd);

  /* Method binSendIndexList: Send a binary representation of IndexList *
   *                          describing message arguments              */
  void binSendIndexList(int sfd);

  /* Method addArgToValList: Add a Val to message ValList */
  void addArgToValList(Val &v) { vl.addArg(v); } ;

  /* Method execute: Execute a message */
  void execute();

  /* Method setNumber: Set the message number */
  void setNumber(int num) { MsgNumber = num; } ; 

  /* Method getNumber: Retrieve the message number */
  int getNumber() { return MsgNumber; } ;

  /* Method operator=: Message assignment */
  Message& operator= ( const Message& m );

  /* Method printOn: Stream output function */
  void printOn(ostream &os = cout);
};

inline ostream &operator<<(ostream &os, Message &obj)
{
	obj.printOn(os);
	return os;
}

#endif
